%% Figures for Testrun 
% Plots controller and disturbance data

% Load Data

close all hidden
clear all

load Data_Time;
load Data_BHA;
load Data_choke;
load Data_MudPump;
load Data_BackpressurePump;
load Data_Position;
load Data_Controller;
load Data_Well;
load Data_Reservoir;
load Data_Observer;

t = 1:1:size(Choke,1);
t = t./1/(1/(Time.inc*Time.decimation));
t2 = 1:1:size(Controller,1);
t2 = t2./1/(1/(Time.inc*Time.decimation));

DController = zeros(size(t,2),size(Controller,2));

ck = 1;
for cj = 1:1:size(Controller,2);
    for ci = Time.decimation:(Time.decimation):size(Controller,1);    
        DController(ck,cj) = Controller(ci,cj); 
        ck = ck+1;
    end
    ck = 1;
end

emod = Choke(:,2) - DController(:,7)*10^-5; % p - p_m
epro = - Choke(:,2) + DController(:,3)*10^-5; % p - r

subplot(4,1,1)
hold on
plot(t2,Controller(:,7)*10^-5,'r');
plot(t2,Controller(:,3)*10^-5,'b--');
plot(t,Choke(:,1),'g-');
plot(t,Choke(:,2),'c--');
hold off
legend('Choke Model p_m','Trajectory p_{c0}','Choke p_c','Choke measured p_{c,n}')
ylabel('[bar]')


subplot(4,1,2)
hold on
plot(t,Choke(:,4)*1000*60,'b');
plot(t,MudPump(:,4)*1000*60,'g');
plot(t,BackPressurePump(:,2)*1000*60,'r');
plot(t,Reservoir(:,1)*1000*60,'c');
plot(t,Observer(:,3)*1000*60,'y--');
plot(t,BHA(:,3)*1000*60,'r--');
hold off
legend('q_c','q_m','q_{bp}','q_{res}','q_{obs}','q_{bit}')
ylabel('[l/min]')

% subplot(4,1,3)
% hold on
% plot(t,Choke(:,5)*100,'r');
% plot(t,Choke(:,6)*100,'g--');
% hold off
% legend('Zc','ZcRef')
% ylabel('[%]')

subplot(4,1,3)
hold on
plot(t2,Controller(:,8)*1000*60,'r');
plot(t,Choke(:,4)*1000*60,'g');
hold off
legend('Flow reference q_{c0}','Flow')
ylabel('[l/min]')



subplot(4,1,4)
hold on
plot(t,BHA(:,2),'g--');
plot(t,BHA(:,1),'b');
plot(t2,Controller(:,2),'y');
plot(t,Observer(:,4)*10^-5,'r');
hold off
legend('P_{bit,n}','P_{bit}','P_{bit,ref}','p_obs')
ylabel('[bar]')
xlabel('time [hours]')

figure()
subplot(3,1,1)
hold on
plot(t2,Controller(:,8)*1000*60,'r');
plot(t,Choke(:,4)*1000*60,'g');
hold off
legend('Flow reference q_{c0}','Flow')
ylabel('[l/min]')

subplot(3,1,2)
hold on
plot(t2,Controller(:,13),'g');
plot(t2,Controller(:,14),'y--');
plot(t2,Controller(:,8)*1000*60,'r--');
hold off
legend('R_u','X','U_c')
xlabel('time [hours]')

subplot(3,1,3)
hold on
plot(t,MudPump(:,1),'r');
plot(t,MudPump(:,2),'b--');
hold off
legend('mp','mpm');


theta = Well(:,7) ./ Well(:,5);
d = theta.*(BackPressurePump(:,2) + BHA(:,3) +  Reservoir(:,1) - Well(:,12));
e = d - theta.*DController(:,6);

k0e = 0.2 ./ theta;

figure()
subplot(4,1,1)
hold on
plot(t2,Controller(:,4),'r');
plot(t2,Controller(:,5),'g');
plot(t2,k0e,'b--');
hold off
legend('k_0','k_1','est')

subplot(4,1,2)
hold on
plot(t2,Controller(:,6));
hold off
legend('k_2')

% subplot(4,1,3)
% hold on
% plot(t,emod,'r');
% plot(t,epro,'g--');
% plot(t,zeros(size(emod,1),1),'--');
% hold off
% legend('e mod','e pro')
% 



subplot(4,1,3)
hold on
plot(t,d,'r-');
plot(t,theta.*DController(:,6),'b');
plot(t,e,'g');
hold off
legend('d','k_2','e')
xlabel('time [hours]')


subplot(4,1,4)
hold on
plot(t2,Controller(:,8)*1000*60,'r-');
plot(t,Choke(:,4)*1000*60,'g');
hold off
legend('Flow reference q_{c0}','Flow')
ylabel('[l/min]')
xlabel('time [hours]')

figure()
subplot(4,1,1)
hold on
plot(t,Well(:,10),'r');
plot(t,Well(:,11),'g');
plot(t,Observer(:,2),'y--');
hold off
legend('rhoa','rhod','rhoaest')

subplot(4,1,2)
hold on
plot(t,Well(:,7),'r');
plot(t,Well(:,8),'g');
hold off
legend('betaa','betad')

subplot(4,1,3)
hold on
plot(t,Well(:,3),'r');
plot(t,Well(:,4),'g');
plot(t,Observer(:,1),'y--');
hold off
legend('Fd','Fa','Faest')
xlabel('time [hours]')

subplot(4,1,4)
hold on
plot(t,Well(:,1),'r');
plot(t,Well(:,2),'g');
plot(t,Well(:,1)+Well(:,2),'y--');
hold off
legend('Ma','Md','M')
xlabel('time [hours]')



figure()
subplot(5,1,1)
plot(t2,Controller(:,9),'r');
legend('Theta')
subplot(5,1,2)
plot(t2,Controller(:,10),'r');
legend('Sigma')
subplot(5,1,3)
plot(t2,Controller(:,11),'r');
legend('omega')
subplot(5,1,4)
hold on
plot(t2,Controller(:,12)*10^-5,'r');
plot(t,BHA(:,1),'b');
hold off
legend('Pbithat','Pbit')
subplot(5,1,5)
hold on
plot(t2,Controller(:,15)*10^-5,'r');
plot(t,Choke(:,1),'b--');
hold off
legend('Pchokehat','Pc')
xlabel('time [hours]')
