function  simulation  = addDrillPipeStand(simulation)
%ADDDRILLPIPESTAND - adds a new stand of drill pipe
%   
%   simulation  = addDrillPipeStand(simulation) adds a new stand of drill pipe,
%   stand length is defined by the pipeStandLength Property. Block position 
%   must be zero for this to occur. After adding pipe blockposition is 
%   reset to to pipeStandLength.
%
%  Input
%   simulation - superstruct
%       .model.Ld - vector of component lengths
%       .model.pipeStandLength - standard pipe stand length
%       .model.bPos - blockposition
%
%   Output
%   simulation - superstruct
%       .model.Ld
%       .model.bPos
%
%   Torbjrn Pedersen / yvind Stamnes 2007

Ld = simulation.model.Ld;
psl = simulation.model.pipeStandLength;
bp = simulation.model.bPos;

if abs(bp)>0
    disp(['Cannot add drill pipe while blockPosition is different than zero, blockPosition = ' num2str(bp)])
    return
end

disp('Adding one stand of drill pipe.')

% find index where drillstring is zero
k0 = find(Ld==0);

% add one pipe stand length to drill string
Ld(k0+1:end)=Ld(k0+1:end) + psl;

simulation.model.Ld = Ld;
simulation.model.bPos = psl;

return