function simulation = calculateDisturbance(simulation,index)
% function which takes in simulation disturbances and runs
% the appropriate disturbance
%
% Input: 
% simulation - superstruct
%   .disturbance.selected

disturbance = simulation.disturbance.type;

modifier = (1/simulation.inc);

% Step reference disturbance
if(disturbance.ref_change)
    
    if(simulation.time > 10*60 && simulation.time < (10*60+0.2))
        simulation.controller.PbitReference = 260;
        simulation.controller.chokeReference = 5*10^5;
    end
    
    if(simulation.time > 15*60 && simulation.time < (15*60+0.2))
        simulation.controller.PbitReference = 250;
        simulation.controller.chokeReference = 10*10^5;
    end
    
    if(simulation.time > 23*60 && simulation.time < (23*60+0.2))
        simulation.controller.PbitReference = 260;
        simulation.controller.chokeReference = 5*10^5;
    end
    
    if(simulation.time > 28*60 && simulation.time < (28*60+0.2))
        simulation.controller.PbitReference = 250;
        simulation.controller.chokeReference = 10*10^5;
    end
    
    if(simulation.time == 3600)
        simulation.controller.PbitReference = 260;
        simulation.controller.chokeReference = 15*10^5;
   
    end

    if(simulation.time == 3600*2)
        simulation.controller.PbitReference = 250;
        simulation.controller.chokeReference = 20*10^5 ;
    end
end

% Ramp reference disturbance
if(disturbance.ref_change1)
    
    if(simulation.time > 500 && simulation.time < 600)
    simulation.controller.PbitReference = simulation.controller.PbitReference + 1/2;
    simulation.controller.chokeReference = simulation.controller.chokeReference + (1/10*10^5);
    end
    
    if(simulation.time > 1200 && simulation.time < 1300)
    simulation.controller.PbitReference = simulation.controller.PbitReference - 1/2;
    simulation.controller.chokeReference = simulation.controller.chokeReference - (1/10*10^5);
    end
end

if(disturbance.saturate)
    
    if(simulation.time > 2*60 && simulation.time < 2*60+0.2)
        simulation.controller.PbitReference = 400;
        simulation.controller.chokeReference = 150*10^5;
    end
    
    if(simulation.time > 3*60 && simulation.time < 3*60+0.2)
        simulation.controller.PbitReference = 280;
        simulation.controller.chokeReference = 40*10^5;
    end
    
end

% Sinusoid reference disturbance
if(disturbance.ref_change2)
    
    simulation.controller.PbitReference = 270 + 20*sin(0.01*simulation.time);
    simulation.controller.chokeReference = 20*10^5 + 10*10^5*sin(0.01*simulation.time);

end

if(disturbance.ref_switch)
    
    simulation.controller.chokeReference = simulation.disturbance.uref(index);

end

if(disturbance.mp_switch)
    
    simulation.mudpump.input = simulation.disturbance.pref(index);

end

if(disturbance.ref_change_sw2)
    
    if(simulation.time > 5*60 && simulation.time < (5*60+0.2))
        simulation.controller.PbitReference = 260;
        simulation.controller.chokeReference = 5*10^5;
    end
    
    if(simulation.time > 10*60 && simulation.time < (10*60+0.2))
        simulation.controller.PbitReference = 250;
        simulation.controller.chokeReference = 10*10^5;
    end
    
    if(simulation.time > 15*60 && simulation.time < (15*60+0.2))
        simulation.controller.PbitReference = 260;
        simulation.controller.chokeReference = 5*10^5;
    end
    
    if(simulation.time > 20*60 && simulation.time < (20*60+0.2))
        simulation.controller.PbitReference = 250;
        simulation.controller.chokeReference = 10*10^5;
    end
    
    if(simulation.time > 25*60 && simulation.time < (25*60+0.2))
        simulation.controller.PbitReference = 260;
        simulation.controller.chokeReference = 5*10^5;
    end
    
    if(simulation.time > 30*60 && simulation.time < (30*60+0.2))
        simulation.controller.PbitReference = 250;
        simulation.controller.chokeReference = 10*10^5;
    end
    
    if(simulation.time > 35*60 && simulation.time < (35*60+0.2))
        simulation.controller.PbitReference = 260;
        simulation.controller.chokeReference = 5*10^5;
    end
    
    if(simulation.time > 40*60 && simulation.time < (40*60+0.2))
        simulation.controller.PbitReference = 250;
        simulation.controller.chokeReference = 10*10^5;
    end
    
    if(simulation.time > 45*60 && simulation.time < (45*60+0.2))
        simulation.controller.PbitReference = 260;
        simulation.controller.chokeReference = 5*10^5;
    end
    
    if(simulation.time > 50*60 && simulation.time < (50*60+0.2))
        simulation.controller.PbitReference = 250;
        simulation.controller.chokeReference = 10*10^5;
    end
    
    if(simulation.time > 55*60 && simulation.time < (55*60+0.2))
        simulation.controller.PbitReference = 260;
        simulation.controller.chokeReference = 5*10^5;
    end
    
    if(simulation.time > 60*60 && simulation.time < (60*60+0.2))
        simulation.controller.PbitReference = 250;
        simulation.controller.chokeReference = 10*10^5;
    end

end

if(disturbance.pterm);
    simulation.model.hbit = simulation.model.hbit + 10*sin(simulation.time/(1*modifier));
    simulation.model.hbitDot = 10*(1/modifier)*cos(simulation.time/(1*modifier));
end

if(disturbance.kick2)
    
    if(simulation.time > 1*60 && simulation.time < 2*60)
        simulation.model.qres = simulation.model.qres + (20/60/1000)/(1/simulation.inc); %simulation.model.qres + 20/60/1000;
    end
    
    if(simulation.time > 3*60 && simulation.time < (3*60+0.2))
        simulation.controller.PbitReference = 260;
        simulation.controller.chokeReference = 5*10^5;
    end
    
    if(simulation.time > 5*60 && simulation.time < (5*60+0.2))
        simulation.controller.PbitReference = 250;
        simulation.controller.chokeReference = 10*10^5;
    end
    if(simulation.time > 6*60 && simulation.time < 7*60)
        
        
        simulation.model.qres = simulation.model.qres - 20/60/1000/(1/simulation.inc);
    end
    
end
    
% Mud loss
if(disturbance.loss2)
    
    if(simulation.time > 1*60 && simulation.time < 2*60)
        simulation.model.qres = simulation.model.qres - (20/60/1000)/(1/simulation.inc); %simulation.model.qres + 20/60/1000;
    end
    
    if(simulation.time > 3*60 && simulation.time < (3*60+0.2))
        simulation.controller.PbitReference = 260;
        simulation.controller.chokeReference = 5*10^5;
    end
    
    if(simulation.time > 5*60 && simulation.time < (5*60+0.2))
        simulation.controller.PbitReference = 250;
        simulation.controller.chokeReference = 10*10^5;
    end
    
    if(simulation.time > 6*60 && simulation.time < 7*60)
        simulation.model.qres = simulation.model.qres + 20/60/1000/(1/simulation.inc);
    end
    
end    


% connection scenario
if(disturbance.loss3)
    
    starttime = 1*60;
    ramptime = 2*60;
    conntime = 10*60;
    
    if(simulation.time > starttime && (simulation.time <= (starttime + ramptime)))
        simulation.mudpump.input =  simulation.mudpump.input - (1/(ramptime*modifier));
    end
    
    if((simulation.time > (starttime + ramptime + conntime)) && (simulation.time <= (starttime + 2*ramptime + conntime)))
        simulation.mudpump.input = simulation.mudpump.input + (1/(ramptime*modifier));
    end
    
    if(simulation.mudpump.input < 0.1)
        simulation.backppump.input = 1;
    else
        simulation.backppump.input = 0.5;
    end
    
    if(simulation.time > 4*60 && simulation.time < 5*60)
        simulation.model.qres = simulation.model.qres - (20/60/1000)/(1/simulation.inc); %simulation.model.qres + 20/60/1000;
    end
    
    if(simulation.time > 10*60 && simulation.time < 11*60)
        simulation.model.qres = simulation.model.qres + 20/60/1000/(1/simulation.inc);
    end
    
end

% Mudpump ramp
if(disturbance.mp_change)
    
    mft = 2*60;
    
    if(simulation.time > 1*60 && simulation.time < 3*60)
         simulation.mudpump.input = simulation.mudpump.input - 1/(2*modifier*mft);
    end
    
    if(simulation.time > 5*60 && simulation.time < 7*60)
         simulation.mudpump.input = simulation.mudpump.input + 1/(2*modifier*mft);
    end
    
    if(simulation.time > 9*60 && simulation.time < 11*60)
         simulation.mudpump.input = simulation.mudpump.input - 1/(2*modifier*mft);
    end
    
    if(simulation.time > 13*60 && simulation.time < 15*60)
         simulation.mudpump.input = simulation.mudpump.input + 1/(2*modifier*mft);
    end
    
    if(simulation.time > 17*60 && simulation.time < 19*60)
         simulation.mudpump.input = simulation.mudpump.input - 1/(2*modifier*mft);
    end
    
    if(simulation.time > 21*60 && simulation.time < 23*60)
         simulation.mudpump.input = simulation.mudpump.input + 1/(2*modifier*mft);
    end
    
    if(simulation.time > 25*60 && simulation.time < 27*60)
         simulation.mudpump.input = simulation.mudpump.input - 1/(2*modifier*mft);
    end
    
    if(simulation.time > 29*60 && simulation.time < 31*60)
         simulation.mudpump.input = simulation.mudpump.input + 1/(2*modifier*mft);
    end
    

end

% Backpressurepump change
if(disturbance.bpp_change)
        
    if(simulation.time == 1200)
         simulation.backppump.input = 1;
    end
end

% Liquid kick pressure difference
if(disturbance.kick1)
    
     if(simulation.time > 1000)
        
         if(simulation.model.BHP < simulation.reservoir.porePressure);
             simulation.model.qres = 20/60/1000 * abs(simulation.model.BHP - simulation.reservoir.porePressure)*10^-5;
         end       
         
     end
     
     
end

% Mud loss pressure difference
if(disturbance.loss1)
   
    if(simulation.time > 1000)
       
        if(simulation.model.BHP > simulation.reservoir.fracPressure);
            simulation.model.qres = - 20/60/1000 * abs(simulation.model.BHP - simulation.reservoir.fracPressure)*10^-5;
        end       
        
    end
    
    
end

% Well change, beta_d rise
if(disturbance.bulkmodulus)
   
    bmst = 0*60; % start time
    bmft = 3*60; % fall/rise time
    bmwt = 11*60; % wait time
    
    if(simulation.time > bmst && simulation.time < (bmst+bmft))
        simulation.model.bulkchange = simulation.model.bulkchange - (3/(8*modifier*bmft));
    end
%     
%     if(simulation.time > bmst+bmwt && simulation.time < (bmst+bmft+bmwt))
%         simulation.model.bulkchange = simulation.model.bulkchange + (3/(8*modifier*bmft));
%     end
end


% Well change, beta_d rise
if(disturbance.bulkmodulus2)
   
    bmst = 12*60; % wait time
    bmft = 1*60; % fall/rise time
    
    if(simulation.time > bmst && simulation.time < (bmst+bmft))
        simulation.model.bulkchange = simulation.model.bulkchange - (4/(8*modifier*bmft*5));
    end
    
    if(simulation.time > 2*bmst && simulation.time < (2*bmst+bmft))
        simulation.model.bulkchange = simulation.model.bulkchange - (4/(8*modifier*bmft*5));
    end

    if(simulation.time > 3*bmst && simulation.time < (3*bmst+bmft))
        simulation.model.bulkchange = simulation.model.bulkchange - (4/(8*modifier*bmft*5));
    end

    if(simulation.time > 4*bmst && simulation.time < (4*bmst+bmft))
        simulation.model.bulkchange = simulation.model.bulkchange - (4/(8*modifier*bmft*5));
    end

    if(simulation.time > 5*bmst && simulation.time < (5*bmst+bmft))
        simulation.model.bulkchange = simulation.model.bulkchange - (4/(8*modifier*bmft*5));
    end
%     
%     if(simulation.time > bmst+bmwt && simulation.time < (bmst+bmft+bmwt))
%         simulation.model.bulkchange = simulation.model.bulkchange + (3/(8*modifier*bmft));
%     end
end


% Well change, rho_d
if(disturbance.muddensity)
   
    mdst = 1*60; % start time
    mdtd = 20*60; % time delay
    mdrt = 30*60; % rise time
    
    if(simulation.time > mdst && simulation.time < (mdst+mdrt))
        simulation.model.rhod = simulation.model.rhod - (100/(modifier*mdrt)); 
    end
    
    if(simulation.time > (mdst+mdtd) && simulation.time < (mdst+mdrt+mdtd))
        simulation.model.rhoa = simulation.model.rhoa - (100/(modifier*mdrt)); 
    end
        
end

% Well change, friction
% Well change, friction
if(disturbance.friction)
   
    if(simulation.time > 12*60 && simulation.time < 12*60+1)
            fwCasing = 0.03;
            fwOpenHole = 0.30; %factor open hole
            simulation.model.fW=[fwCasing fwCasing fwCasing fwOpenHole ]; 
    end
    
    if(simulation.time > 22*60 && simulation.time < 22*60+1)
            fwCasing = 0.03;
            fwOpenHole = 0.40; %factor open hole
            simulation.model.fW=[fwCasing fwCasing fwCasing fwOpenHole ]; 
    end
    
    if(simulation.time > 32*60 && simulation.time < 32*60+1)
            fwCasing = 0.03;
            fwOpenHole = 0.50; %factor open hole
            simulation.model.fW=[fwCasing fwCasing fwCasing fwOpenHole ]; 
    end
    
    if(simulation.time > 42*60 && simulation.time < 42*60+1)
            fwCasing = 0.03;
            fwOpenHole = 0.60; %factor open hole
            simulation.model.fW=[fwCasing fwCasing fwCasing fwOpenHole ]; 
    end
    
    if(simulation.time > 52*60 && simulation.time < 52*60+1)
            fwCasing = 0.03;
            fwOpenHole = 0.70; %factor open hole
            simulation.model.fW=[fwCasing fwCasing fwCasing fwOpenHole ]; 
    end
        
end

% enable drilling
if(disturbance.drill)
   simulation.model.bPosVel = -20/3600; % 20 m / h
end

% enable heave
if(disturbance.heave)
   simulation.model.bPosVel = (5/3600)*sin(0.1*simulation.time); % 20 m / h
end

% connection scenario
if(disturbance.connection)
    
    starttime = 5*60;
    ramptime = 2*60;
    conntime = 11*60;
    
    if(simulation.time > starttime && (simulation.time <= (starttime + ramptime+10)))
        simulation.mudpump.input =  simulation.mudpump.input - (1/(ramptime*modifier));
        simulation.mudpump.input = max(0,simulation.mudpump.input);
    end
    
    if((simulation.time > (starttime + ramptime + conntime)) && (simulation.time <= (starttime + 2*ramptime + conntime + 10)))
        simulation.mudpump.input = simulation.mudpump.input + (1/(ramptime*modifier));
        simulation.mudpump.input = min(1,simulation.mudpump.input);
    end
    
    if(simulation.mudpump.input < 0.1)
        simulation.backppump.input = 1;
    else
        simulation.backppump.input = 0.5;
    end
    
end

if(disturbance.powerLoss)
    
    starttime = 2*60;
    conntime = 5*60;
    
    if(simulation.time > starttime && (simulation.time <= (starttime + 0.2)))
        simulation.mudpump.input =  0;
    end
    
    if((simulation.time > (starttime + conntime)) && (simulation.time <= (starttime + conntime + 0.2)))
        simulation.mudpump.input = 1;
    end
    
    if(simulation.mudpump.input < 0.1)
        simulation.backppump.input = 1;
    else
        simulation.backppump.input = 0.5;
    end
    
end

if(disturbance.changeDynamics1 && simulation.time > 100 && simulation.time <110)
    simulation.idealFlow.frequency = 1;
    simulation.idealFlow.dampingRatio = 0.7;    
end

if(disturbance.changeDynamics2 && simulation.time > 100 && simulation.time <110)
    simulation.idealFlow.frequency = 0.5;
    simulation.idealFlow.dampingRatio = 0.7;    
end

end