function simulation = controlSystem(simulation, index)
% This function runs one iteration of the control system
% the correct controller is selected based on simulation settings
%
% Written by Torbjrn Pedersen 2008

% Calculate referense, either fixed or by trajectory generator
if(simulation.controller.useTrajGen)% && ~simulation.useWeMod
    simulation.controller.chokeReference = trajectoryGenerator(simulation);
end

r = simulation.controller.chokeReference;

if(strcmp(simulation.controller.type,'MRAC'))
    simulation = referenceModel(simulation);
else
    simulation.refmod.chokePressure = nan;
end



if(strcmp(simulation.controller.type,'L1'))
    if(~mod(index,simulation.controller.sampleL1parameters))
    
        if(simulation.l1.chokeControl)
            if(simulation.solver)
                simulation = l1companionmodelChoke(simulation);
                simulation = l1estimatesChoke(simulation);
            else
                simulation = l1companionmodelChokeFixed(simulation);
                simulation = l1estimatesChokeFixed(simulation);
            end
        else
            simulation = l1companionmodel(simulation);
            simulation = l1estimates(simulation);
        end
    end
end


% Get process values
p_p = simulation.measurement.chokePressure;
p_m = simulation.refmod.chokePressure;

% Update errors
simulation.controller.error = simulation.measurement.BHP*10^-5 - simulation.controller.PbitReference; % BHP error
e_p = r - p_p;   % Process error

% Update Parameters

gamma = simulation.controller.gamma;
theta = simulation.model.betaa / simulation.model.Va;
time = simulation.timeSpan;
timeOde = simulation.timeOde;

updateTime = mod(index,simulation.controller.sampleMRACparameters);

if(strcmp(simulation.controller.type,'MRAC'))

if(~updateTime)
% If outside deadband and not saturated, update parameters
deadband = simulation.controller.deadband;

if ((abs(e_p) > deadband) && simulation.controller.idealFlow)
    gain = simulation.controller.gain;
    if (simulation.controller.normalized && (simulation.controller.flowInput ~=0) && (simulation.controller.flowInput ~=15000/60/1000))
        gain = updateLawsNormalized(gain,r,p_p,p_m,gamma,theta,timeOde);
    elseif (~simulation.controller.normalized && (simulation.controller.flowInput ~=0) && (simulation.controller.flowInput ~=15000/60/1000))
        gain = updateLaws(gain,r,p_p,p_m,gamma,theta,timeOde);
    end
    simulation.controller.gain = gain;  
elseif ((abs(e_p) > deadband) && simulation.controller.chokeInput ~= 0 && simulation.controller.chokeInput ~= 1 )
    gain = simulation.controller.gain;
    if simulation.controller.normalized
        gain = updateLawsNormalized(gain,r,p_p,p_m,gamma,theta,time);
    else
        gain = updateLaws(gain,r,p_p,p_m,gamma,theta,time);
    end
    simulation.controller.gain = gain;
end

end

else
simulation.controller.gain = [nan, nan, nan];
end

% Calculate choke gain

%Set samplingFalse true if not at a samplingtime
samplingFalse = mod(index,simulation.controller.sampleController); 

if(samplingFalse)
    return; % No change in gain if this isnt a samling time   
end

switch simulation.controller.type
    case 'PI'
        simulation.controller.error = e_p; % Error choke pressure
        simulation = controllerPIchoke(simulation); % Choke controller
        if(simulation.l1.chokeControl)
            simulation = controllerPIflow(simulation); % Flow controller
        else
            simulation.controller.error = simulation.controller.PbitReference*10^5 - simulation.measurement.BHP;
            simulation = controllerPIflowBHP(simulation); % Flow controller 
        end
        return;
    case 'MRAC'
        simulation.controller.flowInput = controllerMRACpressure(simulation.controller.gain,r,p_p);
        simulation.controller.chokeInput = controllerMRACflow(simulation); 
        return;
    case 'Manual'
    %   Keep previous gain 
        return;
    case 'L1'
        if(simulation.l1.chokeControl)
            if(simulation.solver)
                simulation = l1controlChoke(simulation);
            else
                simulation = l1controlChokeFixed(simulation);
            end
        else
            simulation = l1control(simulation);
        end
        simulation.controller.chokeInput = controllerMRACflow(simulation);
%         count = count + 1
        return;
        
end

% ---------------------------------

end


