function [lbit] = getBitPosition(simulation)
%GETBITPOSITION returns the current bit position in well reference frame
%
%   [lbit] = getBitPosition(simulation)
%
%   Input:
%   simulation - superstruct
%   	.model.Ld - length vector of components
%   	.model.bPos - current block position
%      
%   Output:
%   lbit - bit position [m]
%
%   Torbjrn Pedersen / yvind Stamnes 2008

Ld = simulation.model.Ld;
bp = simulation.model.bPos;
if isempty(bp)
    disp('blockPosition undefined, assuming it to be zero')
    bp=0;
end
LdShift = Ld - bp;
lbit = LdShift(end);

return