function simulation = getChoke(simulation)
% simulation = getChoke(simualtion)
%
% Outputs:
% simulation     - Updated simulation structure
%   .choke.Zc: new choke opening
%   .choke.ZcVel: new choke velocity
% 
% Inputs:
% simualtion    - simulations structure
%   .choke.Zc : previous choke opening
%   .choke.ZcVel: Previous choke velocity
%   .controller.chokeInput: wanted choke opening
%   .timeSpan: numerical timespan for odesolver
%
% Written by Torbjrn Pedersen 2008

% get initial conditions
chokeOpening0           = simulation.choke.Zc;
chokeOpeningVelocity0   = simulation.choke.ZcVel;
controlInput            = simulation.controller.chokeInput;
timeSpan                = simulation.timeSpan;

y0                      = [chokeOpening0 chokeOpeningVelocity0]';

% define A and B matrices

omega = simulation.choke.frequency;
zeta = simulation.choke.dampingRatio;

A = [0 1; -omega^2 -2*zeta*omega];                       
B = [0 ; omega^2];                

ySim = ode4(@(t,y)chokeDynamicsSystem(t,y,A,B,controlInput),timeSpan,y0);

% set chokeOpening and velocity in object

chokeOpening            = ySim(end,1);
chokeOpeningVelocity    = ySim(end,2);

% Crude handling of rate limit
if(abs(chokeOpening - chokeOpening0) > 0.05)
    chokeOpening = chokeOpening0 + 0.05*sign(chokeOpening - chokeOpening0);
    chokeOpeningVelocity = 0;
end

% Handle physical constraints
if chokeOpening < 0
    chokeOpening = 0;
    chokeOpeningVelocity = 0;
elseif chokeOpening > 1
    chokeOpening = 1;
    chokeOpeningVelocity = 0;
end

%Update the simualtion structure
simulation.choke.Zc = chokeOpening;
simulation.choke.ZcVel = chokeOpeningVelocity;

end

%Define the dynamics of the choke
function dydt= chokeDynamicsSystem(t,y,A,B,controlInput)
    dydt = A*y +B*controlInput;
end