function simulation = getIdealFlow(simulation)
% simulation = getIdealFlow(simualtion)
%
% Outputs:
% simulation     - Updated simulation structure
%   .idealFlow.flow: new flow volume
%   .idealFlow.flowChange: new flow change
% 
% Inputs:
% simualtion    - simulations structure
%   .idealFlow.flow : previous flow volume
%   .idealFlow.flowChange: Previous flow change
%   .controller.flowInput: wanted flow volume
%   .timeSpan: numerical timespan for odesolver

% get initial conditions
flow0           = simulation.idealFlow.flow;
flowChange0   = simulation.idealFlow.flowChange;
controlInput            = simulation.controller.flowInput;
timeSpan                = simulation.timeSpan;

y0                      = [flow0 flowChange0]';

% define A and B matrices

omega = simulation.idealFlow.frequency;
zeta = simulation.idealFlow.dampingRatio;

A = [0 1; -omega^2 -2*zeta*omega];                       
B = [0 ; omega^2];                

ySim = ode4(@(t,y)idealFlowDynamicsSystem(t,y,A,B,controlInput),timeSpan,y0);

% set flowOpening and change in object

flow           = ySim(end,1);
flowChange    = ySim(end,2);

% Handle physical constraints
if flow < 0
    flow = 0;
    flowChange = 0;
elseif flow > 15000/60/1000;
    flow = 15000/60/1000;
    flowChange = 0;
end

%Update the simualtion structure
simulation.idealFlow.flow = flow;
simulation.idealFlow.flowChange = flowChange;

end

%Define the dynamics of the flow
function dydt= idealFlowDynamicsSystem(t,y,A,B,controlInput)
    dydt = A*y +B*controlInput;
end