function pump = getPump(pump, timeSpan)
% Adds 1. order dynamics of pump
%
% pump = getPump(pump,timeSpan)
%
% Inputs
% pump     - pumpstruct
%   .rotationalSpeed - initial pump rotational speed
%   .timeConstant    - pump time constant
%   .input           - wanted pump flow {0:1}
% timeSpan  - The time span for the dynamics (used in ode-solver)
%
% Output
% pump      - pumpstruct
%   .rotationalSpeed - new pump rotational speed
%
% Torbjrn Pedersen 2008

y0              = pump.rotationalSpeed;
controlInput    = pump.input;
timeConstant    = pump.timeConstant;

ySim            = ode1(@(t,y)pumpDynamicsSystem(t,y,controlInput, timeConstant),timeSpan,y0);

pump.rotationalSpeed  = ySim(end);
pump.flowRate = pump.Kp * pump.rotationalSpeed;

end

    %Define the dynamics
    function dydt= pumpDynamicsSystem(t,y,controlInput, timeConstant)
        tau = timeConstant; 

        dydt = 1/tau*(-y + controlInput);
    end