function [ depth ] = getWellDepth(simulation,l)
%GETWELLDEPTH - gets the well depth at given length 
%
%   [ depth ] = getWellDepth(simulation,l) interpolates well depth vs. length 
%   data to find depth a input length l. If l is a vector depth is a vector 
%   of same length containing corresponding depths.
%
%   l     - length where depth is wanted, must be given in well reference
%           frame [m]
%   depth - depth at l [m]
%
%   Input
%   Simulation - superstruct
%       .model.Lw - length profile of well
%       .model.Hw - heigth profile of well
%   
%   Output
%       depth - depth vector
%
% yvind Stamnes / Torbjrn Pedersen 2008

Lw = simulation.model.Lw;
Hw = simulation.model.Hw;

depth = interp1(Lw,Hw,l,'linear','extrap');

return