function [u] = innerFlowControl2(qref,simulation)
% controller - gives the desired control input for the choke
% 
% simulation = controller(simulation) 
%       
% INPUTS
% simulation - superstruct
%   .controller.integralError - Previous integral error
%   .controller.epsilon1       - Controller deadband inner
%   .controller.epsilon2       - Controller deadband outer
%   .controller.error  - r - p_c
%   .controller.previousChokeControlInput - last input used
%   .num                      - numerical timestep size
%
% OUTPUTS
% simulation - superstruct
%   .controller.integralError - New integral error
%   .controller.previousChokeControlInput - Updated prev input
%   .controller.chokeInput - New choke input
%
% Written by Torbjrn Pedersen 2008

    
    
    persistent chi;
    persistent time;
    
    if(isempty(chi))
        chi = 0.1;
        time = simulation.time;
    end
    
    steps = simulation.weModSteps;
    ts = [time,(time+(simulation.inc/steps))];

    dp = max(simulation.model.chokePressure - simulation.model.patm,0.001);
    zcref = qref / (0.0046*sqrt((2/1180)*dp));
    zcref = max(zcref,0.005);
    zcref = min(zcref,0.185);
    
    y0 = chi;
    
    [t y] = ode45(@(t,y)defineSystem(t,y,zcref),ts,y0);
    
    chi = max(y(end,1),0.005);
    chi = min(chi,0.185);
    
    u = chi;
    
    time = time + (simulation.inc/steps);
   
end

function dydt = defineSystem(t,y,zcref)

%dydt = zeros(size(y));

dydt = -0.35*y + 0.35*zcref;

end


