function simulation = l1estimates(simulation)
% Updates l1 parameter  estimates 
%
% Input
% pbit - system measured bottomhole pressure
% pbithat - companion model estimated bottomhole pressure
% u - system control input
% time - numerical timestep
%
% Output:
% l1parameters - l1 estimates of parameters for companion model
%
% Written by Torbjrn Pedersen 2008

time = simulation.timeOde;
pbithat = simulation.l1model.pbit;
pbit = simulation.measurement.BHP;
l1parameters = simulation.l1.l1parameters;
l1gains = simulation.l1.l1gains;
l1bounds = simulation.l1.l1bounds;
u = simulation.controller.flowInput;

pbiterror = pbithat - pbit;

y0 = l1parameters;

[t y] = ode45(@(t,y)defineSystem(t,y,pbit,pbiterror,u,l1gains),time,y0);

l1parameters(1) = min(l1bounds(1),y(end,1)); % Upper bound
l1parameters(1) = max(l1bounds(2),l1parameters(1)); % Lower bound

l1parameters(2) = min(l1bounds(3),y(end,2)); % Upper bound
l1parameters(2) = max(l1bounds(4),l1parameters(2));% Lower bound 

l1parameters(3) = min(l1bounds(5),y(end,3)); % Upper bound
l1parameters(3) = max(l1bounds(6),l1parameters(3)); % Lower bound

simulation.l1.l1parameters = l1parameters; 

end

function dydt = defineSystem(t,y,pbit,pbiterror,u,l1gains)

dydt = zeros(size(y));

dydt(1) = -l1gains(1) * pbit * pbiterror;
dydt(2) = -l1gains(2) * pbiterror;
dydt(3) = -l1gains(3) * pbiterror * u;

end