function [simulation] = observer(simulation)

persistent xi0; 
persistent zetahat0;

betad   = simulation.model.betad;
Vd      = simulation.model.Vd;
Md      = simulation.model.Md;
Ma      = simulation.model.Ma;
Fd      = simulation.model.Fd;
rhod    = simulation.model.rhod;
hbit    = simulation.model.hbit;
hbitdot = 0;
g       = simulation.model.g;
M       = Ma+Md;
p_c     = simulation.measurement.chokePressure;
p_p     = simulation.measurement.mudPumpPressure;
q_p     = simulation.measurement.mudPumpFlow;
Fa      = simulation.observer.Fa;
rhoa    = simulation.observer.rhoa;
qbithat = simulation.observer.qbit;
l1      = simulation.observer.l1;
gamma   = simulation.observer.gamma;

theta1est = (Fd+Fa)/M;
theta2est = (rhod-rhoa)*g/M;

if isempty(xi0)
    qbithat0 = 2034/60/1000;
    eta0 =  gamma*[qbithat^3*Vd/(3*l1*betad); -hbit*qbithat*Vd/(l1*betad)];
    thetaest0 = [(Fd+Fa)/M; (rhod-rhoa)*g/M];
    p_p0 = p_p;
    xi0 = [qbithat0 + l1*p_p0];
    zetahat0 = [thetaest0+eta0];
end

y0 = [xi0, zetahat0(1), zetahat0(2)]';

[y, varargout] = ode4m(@(t,y)defineSystem(t, y, Vd, betad, Ma, Md, Fd, q_p, g, hbit, hbitdot, p_c, p_p, theta1est, theta2est, rhod, l1, gamma),simulation.timeSpan2,y0);

simulation.observer.Fa = varargout(end,1).FaEst;
simulation.observer.rhoa = varargout(end,1).rhoaEst;
simulation.observer.qbit = varargout(end,1).qbitEst;
simulation.observer.pbit = varargout(end,1).pbithat;
simulation.observer.qbitdot = varargout(end,1).qbitdothat;
xi0 = y(end,1);
zetahat0 = [y(end,2), y(end,3)]';

simulation.observer.xi = xi0;
simulation.observer.zetahat1 = zetahat0(1);
simulation.observer.zetahat2 = zetahat0(2);
% Should add projection

if(simulation.observer.use)
    simualtion.measurement.BHP = simulation.observer.pbit;
    simulation.measurement.bitFlowRate = simulation.observer.qbit;
end

end

function [dydt, varargout] = defineSystem(t, y, Vd, betad, Ma, Md, Fd, q_p, g, hbit, hbitdot, p_c, p_p, theta1est, theta2est, rhod, l1, gamma)
%rhol 
%dydt = zeros(size(y));

M = Ma + Md;
qbithat = max(0,y(1) - l1*p_p);

if qbithat > 0
qbitdothat = (1/M)*(p_p-p_c)-theta1est*qbithat^2+theta2est*hbit;
else
qbitdothat = max(0,(1/M)*(p_p-p_c)+theta2est*hbit);  
end

pbithat = p_c + Ma*qbitdothat+(M*theta1est-Fd)*qbithat^2 + (rhod*g - M*theta2est)*hbit;

% if qbithat > 0
% dydt(1) = l1*(betad/Vd)*(q_p-qbithat)-theta1est*qbithat^2+theta2est*hbit+(1/M)*(p_p-p_c);
% else
% dydt(1) = max(0,l1*(betad/Vd)*(q_p-qbithat)-theta1est*qbithat^2+theta2est*hbit+(1/M)*(p_p-p_c));
% end
dydt(1) = l1*(betad/Vd)*(q_p-qbithat)+qbitdothat;

eta =  gamma*[qbithat^3*Vd/(3*l1*betad); -hbit*qbithat*Vd/(l1*betad)];
etadotqbit = gamma*[qbithat^2*Vd/(l1*betad); -hbit*Vd/(l1*betad)];
etadothbit = gamma*[0; -qbithat*Vd/(l1*betad)];

% if q_p > 1/60/1000;
dydt(2:3) = -l1*etadotqbit*(betad/Vd)*(q_p - qbithat) + etadotqbit*dydt(1)+etadothbit*hbitdot;
thetahat = y(2:3) - eta;
theta1est = thetahat(1);
theta2est = thetahat(2);
% else
% dydt(2:3) = [0;0];
% end

Fa = min(M*theta1est-Fd, 2.05*10^9*1.25);
Fa = max(Fa, 2.05*10^9*0.75);
rhoa = min(rhod-(M/g)*theta2est,1180*1.25);
rhoa = max(rhoa,1180*0.75);

dydt = [dydt(1); dydt(2); dydt(3)];

if nargout==2
    returnStruct.rhoaEst   = rhoa;
    returnStruct.FaEst  = Fa;
    returnStruct.qbitEst = qbithat;
    returnStruct.pbithat = pbithat;
    returnStruct.qbitdothat = qbitdothat;
    varargout(1) = {returnStruct};
end

end