function simulation = referenceModel(simulation)
% Reference model, generates wanted system behaviour
% by solving the system 
% dy/dt = (r - y);
%
% Input:
% simulation - superstruct
%   .controller.chokeReference - wanted choke pressure
%   .refmod.chokePressure - last model pressure
%
% Output:
% simulation - superstruct
%   .refmod.chokePressure - new model pressure
% 
% Written by Torbjrn Pedersen 2008

r = simulation.controller.chokeReference;

y0 = simulation.refmod.chokePressure;
tau = simulation.refmod.timeconstant;

Am = simulation.controller.MAm;
Bm = simulation.controller.MBm;

% y = ode4(@(t,y)defineSystem(t, y, r, tau), simulation.timeSpan,y0);
[t y] = ode45(@(t,y)defineSystem(t, y, r, Am, Bm), simulation.timeOde,y0);

simulation.refmod.chokePressure  = max(y(end, 1),1*10^5);

end


% function dydt = defineSystem(t, y, r, tau)
% 
% dydt(1) = (1/tau)*(r - y(1));
% 
% end

function dydt = defineSystem(t, y, r, Am, Bm)

dydt = Am*y + Bm*r;

end


