function simulation = simulationInit()
% Runs all the initalization for the simulationStructure
% Note that a WeMod hardware key is required to run WeModForMatlab
% and that flow controllers do not work for WeModForMatlab
% The pregenerated input vectors have a maximum of 10 hours (10*3600)
% sampels and cannot be runned for longer than this
% If WeMod is selected, the WeMod folder has to be the current path

    % Simulation time constants
    simulation.time = 0.1; % simulation start time
    simulation.endtime = 3600*0.01; %3600*24; % simulation end time 
    simulation.inc = 0.1; % simulation controller increment
    simulation.num = simulation.inc*0.1; % controller numerical stepsize
    simulation.updatemodel = 0.1*1/simulation.inc; % how often is the plant updated
    simulation.modelnum = 0.01; % How large is the plant numerical timestep
    simulation.solver = 0; % 0 for fixed 1 for variabel time steps for L1
    
    % Simulator choices
    simulation.useWeMod = 0; % False for Kaasa, true for WeMod (requires WeMod Key, should be runned from wemod folder)
    simulation.input.chokeVector = 0; % True to use pregenerated choke vector
    simulation.input.pumpVector  = 0; % Use pregenerated pump flow vector
    
    % Simulator control choices
    %   simulation.controller.type = 'Manual'; % keeps same control input
%       simulation.controller.type = 'PI';
    simulation.controller.type = 'MRAC';
%     simulation.controller.type = 'L1';
    simulation.controller.idealFlow = 1; % True for q_c control, false for Zc control
    simulation.controller.useTrajGen = 0; % Use trajectory generator for
    simulation.controller.normalized = 1; % use normalized signals for MRAC
    simulation.l1.chokeControl = 1; % Toggle chokecontrol or bit control
    simulation.observer.use = 0; % Use observer for control system
    % choke pressure based on bhp setpoint, doesnt work for weMod
    
    % Sampling time
    %simulation.controller.updateTime = 1; % How often the controller gains are updated
    %simulation.controller.samplingTime = 0.2; % How often the controller measurements are updated
    simulation.controller.sampleController = 1; % How often is the controller updated [1 = each run, 10 = once per 10 runs]
    simulation.controller.sampleL1parameters = 1; % How often is the L1 param updated [1 = each run, 10 = once per 10 runs]
    simulation.controller.sampleMRACparameters = 1; % How often is the MRAC param updated [1 = each run, 10 = once per 10 runs]
    simulation.controller.sampleMeasurements = 1;%(1/simulation.inc)*2; % How often is the measurements updated
    
    % Simulation choke settings
    simulation.choke.Zc = 0.52;
    simulation.choke.Kc = 0.002; %/4; %0.008*0.66; %0.0008
    simulation.choke.ZcVel = 0;
    simulation.choke.frequency = 1; %2*pi/5;
    simulation.choke.dampingRatio = 0.7;
    [simulation.choke.flowTable,simulation.choke.areaTable] = makeChokeChar(simulation);
    
    % Simulation mudpump settings
    simulation.mudpump.input = 1; %inital control signal
    simulation.mudpump.rotationalSpeed = simulation.mudpump.input;
    simulation.mudpump.Kp = 2000/60/1000; % 2000 liters/min
    simulation.mudpump.timeConstant = 15;
    
    % Simulation backpressurepump settings
    simulation.backppump.input = 0.5; %inital control signal
    simulation.backppump.rotationalSpeed = simulation.backppump.input;
    simulation.backppump.Kp = 400/60/1000; % 400 liters/min
    simulation.backppump.timeConstant = 10;
    
    % Simulation disturbance settings 
    % logic panel set to trigger disturbances in disturbance file
    
    % Reference changes
    simulation.disturbance.type.ref_change = 1; % Steps
    simulation.disturbance.type.ref_change1 = 0; % ramps
    simulation.disturbance.type.ref_change2 = 0; % sinusoid
    simulation.disturbance.type.ref_switch = 0; % Square wave
    simulation.disturbance.type.ref_change_sw2 = 0; % Different square wave
    simulation.disturbance.type.saturate = 0; % high step
    
    % Pump disturbances
    simulation.disturbance.type.mp_change = 0; % mud pump change
    simulation.disturbance.type.mp_switch = 0; % mud pump switch signal
    simulation.disturbance.type.bpp_change = 0;
    
    % Create Switching Data
    
    if(simulation.disturbance.type.ref_switch)
        [pref,uref] = generateRefSignals(simulation);
        simulation.disturbance.pref = pref;
        simulation.disturbance.uref = uref;
    end
    
    % Reservervoir disturbances
    simulation.disturbance.qresEnabled = 0; % Not tested after update
    simulation.disturbance.continousMudLossEnabled = 0; % Not tested after update
    simulation.disturbance.type.kick1 = 0; % Not tested after update
    simulation.disturbance.type.loss1 = 0; % Not tested after update
    simulation.disturbance.type.loss2 = 0; % Loss scenario high flow
    simulation.disturbance.type.loss3 = 0; % Loss scenario low flow
    simulation.disturbance.type.kick2 = 0; % kick scenario
    
    % Parameter changes
    simulation.disturbance.type.bulkmodulus = 0; % change beta scenario
    simulation.disturbance.type.bulkmodulus2 = 0; %  change beta 2 scenario
    simulation.disturbance.type.muddensity = 0;  %mud density change
    simulation.disturbance.type.friction = 0; % friction steps

    % Choke
    simulation.disturbance.type.nonlinearChoke = 0; %use nonlinear choke
    simulation.disturbance.type.dynamics = 0; % dynamics on (normal)
    simulation.disturbance.type.changeDynamics1 = 0; % slow dynamics
    simulation.disturbance.type.changeDynamics2 = 0; % slower dynamics
    
    % String motion
    simulation.disturbance.type.drill = 0; % drillspeed
    simulation.disturbance.type.heave = 0; % heave
    simulation.disturbance.type.pterm = 0; % fast pressure term
    
    % Scenarios
    simulation.disturbance.type.connection = 1; % connection
    simulation.disturbance.type.powerLoss = 0; % power loss
    
    % Measurement noise
    simulation.measurement.enableNoise = 0; % Turn on/off noise (setting is bad for bhp, to large noise)
    simulation.measurement.generateNewNoiseVector = 1; % if true, create new noise vector
    simulation.measurement.filter = 0; % turn on/off low pass filter
    simulation.measurement.size = 1; % Percent noise added (0 - 100), std deviation;
    simulation.filter.timeconstant = 1.8;
    
    if(simulation.measurement.generateNewNoiseVector && simulation.measurement.enableNoise)
       generateNoiseVector(simulation.endtime*(1/simulation.inc),simulation.measurement.size); 
    end
  
    % Reference model
    if(simulation.useWeMod)
        simulation.refmod.chokePressure = 30.2*10^5; % Inital value    
    else
    	simulation.refmod.chokePressure = 10.2*10^5; % Inital value
    end
    simulation.refmod.chokePressure = 10.2*10^5; % Inital value
    simulation.refmod.timeconstant = 1; % changes how fast the reference model is, higher is slower
    
    
    % BHP controller PI gains ( Not used atm)
    simulation.controller.Ti = (1300)*0.001;
    simulation.controller.Kp = 0.001;
    
    if(simulation.useWeMod)
        simulation.controller.PbitReference = 800;    
    else
    	simulation.controller.PbitReference = 250;
    end
    
    simulation.controller.aChoke = 0.1;
     % Scaled in controller
    
    % Choke Controller PI gains

    if(simulation.useWeMod)
        simulation.controller.chokeReference = 30*10^5;  
    else
    	simulation.controller.chokeReference = 10*10^5;
    end
    %simulation.controller.chokeReference = 10*10^5;
    simulation.controller.chokeInput = simulation.choke.Zc;
    simulation.controller.Ti2 = 0.25;
    simulation.controller.Kp2 = -5*10^-8;
    
    % Flow Controller PI gains
    if(simulation.l1.chokeControl)
        simulation.controller.PIflowKp = -2.05*10^-8;
        simulation.controller.PIflowTI = 3.0;%4.25;
    else
        simulation.controller.PIflowKp = -2.64*10^-8;
        simulation.controller.PIflowTI = 8.0;%4.25;
    end
    
    
    if(simulation.useWeMod)
        simulation.controller.flowInput = 0/60/1000;    
    else
    	simulation.controller.flowInput = 2223.3/60/1000;
    end
    
    % Shared PI settings
    simulation.controller.integralError = 0;
    simulation.controller.integralErrorFlow = 0;
    simulation.controller.previousFlowControlInput = simulation.controller.flowInput; 
    simulation.controller.previousChokeControlInput = simulation.choke.Zc; % previous control input
    simulation.controller.error = []; % initial error
    simulation.controller.enablePI = 1; % start flag
    simulation.controller.enablePIflow = 1; % start flag
    simulation.controller.ucb = simulation.controller.chokeInput; % bias
    simulation.controller.ucbFlow = simulation.controller.flowInput;
    simulation.controller.epsilon1 = 0.05; % inner deadband limit
    simulation.controller.epsilon2 = 0.3; % outer deadband limit
    
    % Simulations l1 settings
   
  
    if(simulation.l1.chokeControl)
        sigma_upper = 1.32*10^7;
        sigma_lower = -4.5*10^6;
        sigma_0 = 8.95*10^5; %-1.2537*10^9; %2*10^5; %;
        theta_upper = 2.5;
        theta_lower = 0.1;
        theta_0 = 0.3;
        omega_upper = -1*10^5;
        omega_lower = -2*10^8;
        omega_0 = -2.44*10^7;%1500; % 6500;
        gamma_theta = 0*1000*10^-4; %*10^-18;
        gamma_sigma = 5*10^5;
        gamma_omega = 5*10^5;
    else
        sigma_upper = 1.45*10^10;
        sigma_lower = -8.8*10^9;
        sigma_0 = 5.76*10^5; %-1.2537*10^9; %2*10^5; %;
        theta_upper = 0.5;
        theta_lower = 0.1;
        theta_0 = 0.3;
        omega_upper = -2500;
        omega_lower = -1.5*10^9;
        omega_0 = -1.82*10^7;%1500; % 6500;
        gamma_theta = 0*1000*10^-8;
        gamma_sigma = 5*10^5;
        gamma_omega = 5*10^5; 
    end
    
    l1parameters = [theta_0,sigma_0,omega_0]';
    l1gains = [gamma_theta, gamma_sigma, gamma_omega];
    l1bounds = [theta_upper,theta_lower,sigma_upper,sigma_lower,omega_upper,omega_lower];
    simulation.l1.l1k = -5*10^-8;
    simulation.l1.l1Am = -0.3;
    simulation.l1.l1kChoke = -5*10^-8;
%     simulation.l1.l1kChoke = 0.0085; %1/133;
    simulation.l1.l1AmChoke = -0.3;
    simulation.l1.l1parameters = l1parameters;
    simulation.l1.l1gains = l1gains;
    simulation.l1.l1bounds = l1bounds;
    simulation.l1.l1boundsChoke = l1bounds;
    simulation.l1.filter = tf([1],[1 0]);
    simulation.l1.ubounds = [15000/60/1000, 0];
    simulation.controller.l1X = 2200/60/1000;
    pbithat_upper = 1500*10^5;
    pbithat_lower = 1*10^5;
    simulation.l1model.bounds = [pbithat_upper,pbithat_lower];
    simulation.controller.unfilteredu = 0;
    

   
    % Ideal Flow Control settings
    simulation.idealFlow.flow = 2223.3/60/1000;
    simulation.idealFlow.flowChange = 0; 
    simulation.idealFlow.frequency = 4;
    simulation.idealFlow.dampingRatio = 0.7; 
    
    % MRAC inital gains
    %simulation.controller.gain = [8.1943*10^-9 8.1943*10^-9 1*2200/60/1000]'; %Init gain
    simulation.controller.gain = [1.6389*10^-8 1.6389*10^-8 1*2200/60/1000]'; %Init gain
    
    if(simulation.controller.normalized)
        simulation.controller.gamma = [1*10^-14, 1*10^-14, 5*10^-9]'; %adaptation
    else
        simulation.controller.gamma = [1*10^-18, 1*10^-18, 1*10^-10]'; %adaptation
    end
    
    simulation.controller.deadband = 0*10^5;
    
    simulation.controller.MAm = -0.2;
    simulation.controller.MBm = 0.2;
     
    % Observer
    
    simulation.observer.Fa = 2.05*10^9;
    simulation.observer.rhoa = 1180;
    simulation.observer.qbit = 2000/60/1000;
    simulation.observer.pbit = 247.7*10^5;
    simulation.observer.l1 = 1*10^-7; %1/6000;
    simulation.observer.gamma = diag([1*10^6 1*10^-7]);
    simulation.observer.xi = nan;
    simulation.observer.zetahat1 = nan;  
    simulation.observer.zetahat2 = nan;
    simulation.observer.qbitdot = 0;
    
    % Drill string
    
    simulation.model.pipeStandLength = 27; % pipe stand length
    
    in2m=0.0254;    % [m/in] Conversion factor 

    Ld0=100;        % [m] Hose/pipe length from pump to top drive
    Ld1=135*27;     % [m] 5+1/2" Drill string
    Ld2=Ld1+35*27;  % [m] 5" Drill string
    LdBHA=Ld2+130;  % [m] BHA
    LdBit=LdBHA+1;  % [m] Bit
   
    simulation.model.Ld=[-Ld0 0 Ld1 Ld2 LdBHA LdBit];  % length vector

    % inner diameter
    simulation.model.Ddi=[5 4+2/3 4+8/29 3 1]*in2m;
    % outer diameter
    simulation.model.Ddo=[NaN 5.5 5.0 6.0 8.0]*in2m;
    
    % friction factors
    fdHose=1.0; % factor in hoses topside including topdrive and bypass actuator (for sending downlink)
    fdPipe=0.03; % [no unit] factor drillstring, inner
    fdBit=5; % factor drillstring in bit
    fdBHA=3;  % factor drill string BHA
    
    simulation.model.fDi = [fdHose fdPipe fdPipe fdPipe fdBit]; % friction factors, innside
    simulation.model.fDo= [NaN fdPipe fdPipe fdBHA fdBHA];  % friction factors, outside
    
    % input block position
    simulation.model.bPos = 0;   % [m] block position, 0 min, 27 max
    simulation.model.bPosVel = 0;%-20/3600; % 20 m/h [m/s]
    % Well
   
    simulation.model.Lw=[0 346 1619 3800 5400];  % length
    simulation.model.lw = 4721; % initial length of well
    simulation.model.casingShoePos = simulation.model.Lw(4); %casingShoePosition

    % vertical height
    simulation.model.Hw=[0 346 1162 1826 1826];

    % (inner) diameter 
    simulation.model.Dw=[9+2/3 9+2/3 9+2/3 8.5]*in2m;

    % friction factors well
    fwCasing=fdPipe; % factor casing
    fwOpenHole=0.03; %factor open hole

    simulation.model.fW=[fwCasing fwCasing fwCasing fwOpenHole ];   

    % model parameters
    simulation.model.betad = 3.5*10^9;%2.5*10^6; %[Pa]%25000; %14000;
    simulation.model.betag = 1.42*10^5; % Pa , 200;
    simulation.model.bulkchange = 0.5;
    simulation.model.VaDot = 0; % m/s
    simulation.model.rhod = 1180; % SG = 1.18 g/cm^3
    simulation.model.rhoa = 1180; % SG = 1.18 g/cm^3
    simulation.model.rhogIn = 1.618;   % kg/m^3;
    simulation.model.rhogOut = 1.2015; % kg/m^3;
    simulation.model.rhoaChange = 1.0;   % multiplication factor
    simulation.model.g = 9.81;  % m/s^2
    simulation.model.patm = 1*10^5; % Pa
    
    % reservoir pressure parameters
    simulation.reservoir.fracPressure = 260*10^5;
    simulation.reservoir.porePressure = 230*10^5;
    simulation.reservoir.mudLossConstant = 2*10^-10;
    simulation.reservoir.continousMudLossConstant = 1*10^-10;
    simulation.reservoir.kickConstant = 2*10^-10;
    
    % models inital conditions
    simulation.model.mudPumpPressure = 190*10^5; % Pa
    simulation.model.bitFlowRate = 2023.3/60/1000; % m^3/s
    simulation.model.chokePressure = 10.2*10^5; % Pa
    simulation.model.massOfGas = 0; % kg/m^3
    simulation.model.BHP = 247.7*10^5; % Pa
    simulation.l1model.pbit = simulation.model.BHP;
    simulation.l1model.pchoke =  simulation.model.chokePressure;
    simulation.model.qres = 0; % m^3/s
    simulation.model.massGas = 0;
    
    % Calculate initial geometry
    simulation = updateCurrentGeometry(simulation);
    simulation = updateGeometry(simulation);

    
end
   