function simulation = takeMeasurement(simulation,index)
% function takeMeasurement add noise and sampling effects to
% the system, the function uses pre-generated noise vectors to
% add a predefined amount of noise to the the "real" values from the
% system
%
% function simulation = takeMeasurement(simulation)
% 
% Input:
% simulation - superstruct
%   .model.BHP
%   .model.bitFlowRate
%   .model.chokePressure
%   .model.chokeFlow
%   .model.mudPumpPressure
%   .model.mudPumpFlow
%
% Output:
% simulation - superstruct
%   .measurement.BHP 
%   .measurement.bitFlowRate
%   .measurement.chokePressure
%   .measurement.chokeFlow
%   .measurement.mudPumpPressure
%   .measurement.mudPumpFlow
%
% Torbjrn Pedersen 2008

    persistent noise;
    persistent signal;
    persistent hold;

    if isempty(noise)
        load('noise.mat','noise');
        noise = noise;
    end
    
    if isempty(hold);
        hold = [simulation.model.BHP;
            simulation.model.bitFlowRate;
            simulation.model.chokePressure;
            simulation.model.chokeFlow;
            simulation.model.mudPumpPressure;
            simulation.mudpump.flowRate;
            simulation.backppump.flowRate];
    end
    
    sampleFalse = mod(index,simulation.controller.sampleMeasurements);
    
    if(sampleFalse)
        simulation.measurement.BHP = hold(1);
        simulation.measurement.bitFlowRate = hold(2);
        simulation.measurement.chokePressure = hold(3);
        simulation.measurement.chokeFlow = hold(4);
        simulation.measurement.mudPumpPressure = hold(5);
        simulation.measurement.mudPumpFlow = hold(6);
        simulation.measurement.backPPumpFlow = hold(7);
        return;
        
    else

        if(~simulation.measurement.enableNoise)
            
            simulation.measurement.BHP = simulation.model.BHP;
            simulation.measurement.bitFlowRate = simulation.model.bitFlowRate;
            simulation.measurement.chokePressure = simulation.model.chokePressure;
            simulation.measurement.chokeFlow = simulation.model.chokeFlow;
            simulation.measurement.mudPumpPressure = simulation.model.mudPumpPressure;
            simulation.measurement.mudPumpFlow = simulation.mudpump.flowRate;
            simulation.measurement.backPPumpFlow = simulation.backppump.flowRate;
            %return   
            
        else
            
            % If not add noise 
            %t = simulation.time;
            simulation.measurement.BHP = simulation.model.BHP * noise.BHP(index);
            simulation.measurement.bitFlowRate = simulation.model.bitFlowRate * noise.bitFlowRate(index);
            simulation.measurement.chokePressure = simulation.model.chokePressure * noise.chokePressure(index);
            simulation.measurement.chokeFlow = simulation.model.chokeFlow * noise.chokeFlow(index); 
            simulation.measurement.mudPumpPressure = simulation.model.mudPumpPressure * noise.mudPumpPressure(index);
            simulation.measurement.mudPumpFlow = simulation.mudpump.flowRate * noise.mudPumpFlow(index);
            simulation.measurement.backPPumpFlow = simulation.backppump.flowRate * noise.backPPumpFlow(index);

            if(simulation.measurement.filter)

                if isempty(signal)
                   signal = [simulation.measurement.BHP;
                       simulation.measurement.bitFlowRate;
                       simulation.measurement.chokePressure;
                       simulation.measurement.chokeFlow;
                       simulation.measurement.mudPumpPressure;
                       simulation.measurement.mudPumpFlow;
                       simulation.measurement.backPPumpFlow;
                       ];       
                end

                simulation.measurement.BHP = signalFilter(simulation,simulation.measurement.BHP,signal(1));
                simulation.measurement.bitFlowRate = signalFilter(simulation,simulation.measurement.bitFlowRate,signal(2));
                simulation.measurement.chokePressure = signalFilter(simulation,simulation.measurement.chokePressure,signal(3));
                simulation.measurement.chokeFlow = signalFilter(simulation,simulation.measurement.chokeFlow,signal(4));
                simulation.measurement.mudPumpPressure = signalFilter(simulation,simulation.measurement.mudPumpPressure,signal(5));
                simulation.measurement.mudPumpFlow = signalFilter(simulation,simulation.measurement.mudPumpFlow,signal(6));
                simulation.measurement.backPPumpFlow = signalFilter(simulation,simulation.measurement.backPPumpFlow,signal(7));

                signal = [simulation.measurement.BHP;
                   simulation.measurement.bitFlowRate;
                   simulation.measurement.chokePressure;
                   simulation.measurement.chokeFlow;
                   simulation.measurement.mudPumpPressure;
                   simulation.measurement.mudPumpFlow;
                   simulation.measurement.backPPumpFlow;
                ];  

            end
        end
    end
    
    %update last
    hold = [simulation.measurement.BHP;
            simulation.measurement.bitFlowRate;
            simulation.measurement.chokePressure;
            simulation.measurement.chokeFlow;
            simulation.measurement.mudPumpPressure;
            simulation.measurement.mudPumpFlow;
            simulation.measurement.backPPumpFlow];
end