function gain = updateLawsNormalized(gain,r,p_p,p_m,gamma,theta,time)
% Updates the gain vector the controller based on
% the error between the process and the reference model and
% the normalized input signals
% The gain is bounded between fixed limits
% 
% Input
% gain - previous gain
% r - model reference (p_m)
% p_p - process pressure (p_c)
% gamma - adaptation gain parameters
% time - numerical timestep
%
% Output:
% gain - controller gain vector
%
% Written by Torbjrn Pedersen 2008

e = p_p - p_m;

y0 = gain;

[t y] = ode45(@(t,y)defineSystem(t,y,e,r,p_p,gamma),time,y0);

gain(1) = min(1*10^-6,y(end,1)); % min(-0.1*10^-10,y(end,1));
gain(1) = max(1*10^-10,gain(1));
gain(2) = min(1*10^-6,y(end,2)); %min(0.1,max(0.1*10^-10,y(end,2)));
gain(2) = max(1*10^-10,gain(2));
gain(3) = max(-2000/60/1000,y(end,3)); %min(3200/60/1000,y(end,3));
gain(3) = min(15000/60/1000,gain(3));

end


function dydt = defineSystem(t,y,e,r,p_p,gamma)

%dydt = zeros(size(y));

dydt(1) = -(gamma(1)) * e * r * (1/(1+(r^2)));
dydt(2) = (gamma(2)) * e * p_p * ((1)/(1+(p_p^2)));
dydt(3) = (gamma(3)) * e;

dydt = [dydt(1), dydt(2), dydt(3)]';


end